'use strict';

/**
 * Stripe Card Element script.
 * @version 1.0.0
 * @author OLD-ONE
 * @copyright OLD-ONE
 * @see stripe.elements(options?) https://stripe.com/docs/js/elements_object/create_without_intent
 * @see Card Element and Payment Element https://stripe.com/docs/payments/payment-card-element-comparison
 * @see elements.create("card", options) https://stripe.com/docs/js/elements_object/create_element?type=card
 * @see Element style https://stripe.com/docs/js/appendix/style?type=card
 */
// Stripe public key
const stripePK = 'ここにStripeで取得した public key を設定';

// Server api url
const siteURL = location.href.replace(/^(.+)\/[^\/]*$/u, '$1');
const fetchURL = siteURL + '/card_element.php';

// After payment message
const checkStatusMessage = {
  usd: {
    succeeded: 'Payment succeeded!',
    processing: 'Your payment is processing.',
    requires_payment_method: 'Your payment was not successful, please try again.',
    requires_capture: 'Payment verification succeeded!',
    requires_action: 'Your payment procedure succeeded!',
    default: 'Something went wrong.'
  },
  jpy: {
    succeeded: 'お支払いが完了しました。',
    processing: 'お支払いは処理中です。',
    requires_payment_method: 'お支払いは正常に完了しませんでした。もう一度お試しください。',
    requires_capture: 'お支払い認証が完了しました。',
    requires_action: 'お支払い手続きが完了しました。',
    default: '何か問題が発生しました。'
  }
};

// Form element
const form = document.getElementById('card-form');

// Stripe order description
const description = 'Stripe order';

// Amount and currency
const amount = parseInt(form.amount.value, 10);
const currency = (form.currency) ? form.currency.value : 'jpy';

// Capture_method    Default is automatic.    [ automatic | automatic_async | manual ]
const capture_method = 'automatic';

// Set up future usage    [ null | off_session | on_session ]    https://stripe.com/docs/api/payment_intents/object#payment_intent_object-setup_future_usage
const setup_future_usage = null;

// URL where the customer should be redirected after the PaymentIntent is confirmed
const returnURL = location.href;
const addQueryAfterSubmit = true; // If this value is set to true, display checkStatusMessage after submit.

// Stripe instance
const stripe = Stripe(stripePK);

// Stripe elements options
const elementsOption = {
  mode: 'payment', // [ payment | setup | subscription ]
  amount: amount,
  currency: currency,
  automatic_payment_methods: {
    enabled: true, // Default is true
  }
};

// Stripe elements instance
const elements = stripe.elements(elementsOption);

// Stripe card element styles and classes
const styles = {
  base: {
    height: '42px',
    padding: '2px',
    iconColor: '#0570de',
    color: '#444',
    fontWeight: '500',
    fontFamily: 'sans-serif',
    fontSize: '16px',
    letterSpacing: '0',
    fontSmoothing: 'antialiased',
    lineHeight: '42px',
    ':-webkit-autofill': {
      color: '#fce883'
    },
    '::placeholder': {
      color: '#6d6e78'
    }
  },
  invalid: {
    iconColor: '#df1b41',
    color: '#df1b41'
  }
};
const classes = {
  focus: 'focused',
  empty: 'empty',
  invalid: 'invalid'
};

// Stripe card element options
const options = {
  style: styles,
  classes: classes,
  hidePostalCode: true
};

// Create card element
const cardElement = elements.create('card', options);

// Add event listener after DOMContentLoaded
window.addEventListener('DOMContentLoaded', () => {
  cardElement.mount('#card-element');
  cardElement.addEventListener('change', (event) => {
    if (event.error) handleError(event.error);
  });
  form.addEventListener('submit', (event) => {
    handleSubmit(event);
  });
  checkStatus();
});

// Error action
const handleError = (error) => {
  const errorMessage = error.message ?? 'An unexpected error occurred.';
  showMessage(errorMessage);
};

// Submit action
const handleSubmit = async (event) => {
  event.preventDefault();
  if (!stripe) {
    // Stripe.js hasn't yet loaded.
    // Make sure to disable form submission until Stripe.js has loaded.
    return;
  }
  setLoading(true);
  // Create the SetupIntent and obtain clientSecret
  const res = await fetch(fetchURL, {
    method: 'POST',
    headers: {'Content-Type': 'application/json'},
  });
  const {client_secret: clientSecret} = await res.json();
  const {error} = await stripe.confirmCardSetup(clientSecret, {
    payment_method: {
      card: elements.getElement('card'),
    }
  });
  if (error) {
    handleError(error);
    setLoading(false);
  } else {
    // Create the SetupIntent and obtain clientSecret
    const data = {
      description: (form.description && form.description.value) ? form.description.value : description,
      amount: parseInt(form.amount.value, 10),
      currency: currency,
      capture_method: capture_method
    };
    data.city = (form.city && form.city.value) ? form.city.value : null;
    data.country = (form.country && form.country.value) ? form.country.value : null;
    data.line1 = (form.line1 && form.line1.value) ? form.line1.value : null;
    data.line2 = (form.line2 && form.line2.value) ? form.line2.value : null;
    data.postal_code = (form.postal_code && form.postal_code.value) ? form.postal_code.value : null;
    data.state = (form.state && form.state.value) ? form.state.value : null;
    data.email = (form.email && form.email.value) ? form.email.value : null;
    data.name = (form.name && form.name.value) ? form.name.value : null;
    data.phone = (form.phone && form.phone.value) ? form.phone.value : null;
    const address = (data.city || data.country || data.line1 || data.line2 || data.postal_code || data.state) ? {} : null;
    if (address !== null) {
      address.city = data.city;
      address.country = data.country;
      address.line1 = data.line1;
      address.line2 = data.line2;
      address.postal_code = data.postal_code;
      address.state = data.state;
    }
    const res = await fetch(fetchURL, {
      method: 'POST',
      headers: {'Content-Type': 'application/json'},
      body: JSON.stringify(data)
    });
    const {client_secret: clientSecret, payment_intent_id: paymentIntentId} = await res.json();
    const confirmCardPaymentArgs = {
      payment_method: {
        card: cardElement,
        billing_details: {
          address: address,
          email: data.email,
          name: data.name,
          phone: data.phone
        }
      },
      setup_future_usage: setup_future_usage,
      // Return URL where the customer should be redirected after the PaymentIntent is confirmed.
      return_url: returnURL,
      receipt_email: data.email
    };
    stripe.confirmCardPayment(clientSecret, confirmCardPaymentArgs).then((result) => {
      if (result.error) {
        // Inform the customer that there was an error.
        handleError(result);
      } else {
        stripePaymentIntentId(paymentIntentId, result.paymentIntent);
      }
    });
  }
};

// Append payment intent id to form and submit form
const stripePaymentIntentId = (paymentIntentId, paymentIntent) => {
  const hiddenInput = document.createElement('input');
  hiddenInput.setAttribute('type', 'hidden');
  hiddenInput.setAttribute('name', 'payment_intent');
  hiddenInput.setAttribute('value', paymentIntentId);
  form.appendChild(hiddenInput);
  if (addQueryAfterSubmit) {
    const query = '?payment_intent=' + paymentIntentId + '&payment_intent_client_secret=' + paymentIntent.client_secret + '&redirect_status=' + paymentIntent.status;
    form.action = form.action.replace(/\?[^\?]*$/ui, '') + query;
  }
  form.submit();
};

// Show messages
const showMessage = (messageText, duration) => {
  duration = duration || 4000;
  const messageContainer = document.querySelector('#payment-message');
  messageContainer.classList.remove('hidden');
  messageContainer.textContent = messageText;
  setTimeout(() => {
    messageContainer.classList.add('hidden');
    messageContainer.textContent = '';
  }, duration);
};

// Show a spinner on payment submission
const setLoading = (isLoading) => {
  if (isLoading) {
    // Disable the button and show a spinner
    document.querySelector('#submitBtn').disabled = true;
    document.querySelector('#spinner').classList.remove('hidden');
    document.querySelector('#button-text').classList.add('hidden');
  } else {
    document.querySelector('#submitBtn').disabled = false;
    document.querySelector('#spinner').classList.add('hidden');
    document.querySelector('#button-text').classList.remove('hidden');
  }
}

// Fetches the payment intent status after payment submission
const checkStatus = async () => {
  const clientSecret = new URLSearchParams(window.location.search).get('payment_intent_client_secret');
  if (!clientSecret) {
    return;
  }
  const {paymentIntent} = await stripe.retrievePaymentIntent(clientSecret);
  const messageLang = checkStatusMessage[currency] || checkStatusMessage.en;
  const message = messageLang[paymentIntent.status] || messageLang.default;
  showMessage(message);
};
